@@ -28,10 +28,10 @@
 
 #include "common.h"
 #include "compat.h"
-#include "settings.h"
 #include "bookmark.h"
-#include "window.h"
+#include "settings.h"
 #include "backendgvfs.h"
+#include "window.h"
 #include "bookmarkeditdialog.h"
 
 
@@ -43,6 +43,7 @@ typedef struct _GigoloBookmarkEditDialogPrivate			GigoloBookmarkEditDialogPrivat
 struct _GigoloBookmarkEditDialogPrivate
 {
 	GigoloWindow *parent;
+	gint dialog_type;
 
 	GtkWidget *table;
 
@@ -61,6 +62,12 @@ struct _GigoloBookmarkEditDialogPrivate
 	GtkWidget *host_label;
 	GtkWidget *host_entry;
 
+	GtkWidget *folder_label;
+	GtkWidget *folder_entry;
+
+	GtkWidget *path_label;
+	GtkWidget *path_entry;
+
 	GtkWidget *port_label;
 	GtkWidget *port_spin;
 
@@ -77,13 +84,19 @@ struct _GigoloBookmarkEditDialogPrivate
 
 	GigoloBookmark *bookmark_init;
 	GigoloBookmark *bookmark_update;
+
+	gulong browse_host_signal_id;
 };
 
 static void gigolo_bookmark_edit_dialog_set_property		(GObject *object, guint prop_id,
 															 const GValue *value, GParamSpec *pspec);
+static void browse_host_finished_cb							(G_GNUC_UNUSED GigoloBackendGVFS *bnd,
+															 GSList *shares,
+															 GigoloBookmarkEditDialog *dialog);
 
 
-struct MethodInfo {
+struct MethodInfo
+{
 	const gchar *scheme;
 	guint port;
 	guint flags;
@@ -103,7 +116,9 @@ enum {
 	SHOW_PORT      = 0x00000020,
 	SHOW_USER      = 0x00000040,
 	SHOW_DOMAIN    = 0x00000080,
-	SHOW_DEVICE    = 0x00000100
+	SHOW_DEVICE    = 0x00000100,
+	SHOW_FOLDER    = 0x00000200,
+	SHOW_PATH      = 0x00000400
 };
 
 enum {
@@ -124,11 +139,11 @@ enum {
 };
 
 static struct MethodInfo methods[] = {
-	{ "ftp",  21,	SHOW_PORT | SHOW_USER },
-	{ "sftp", 22,	SHOW_PORT | SHOW_USER },
-	{ "smb",  0,	SHOW_SHARE | SHOW_USER | SHOW_DOMAIN },
-	{ "dav",  80,	SHOW_PORT | SHOW_USER },
-	{ "davs", 443,	SHOW_PORT | SHOW_USER },
+	{ "ftp",  21,	SHOW_PORT | SHOW_USER | SHOW_FOLDER },
+	{ "sftp", 22,	SHOW_PORT | SHOW_USER | SHOW_FOLDER },
+	{ "smb",  0,	SHOW_SHARE | SHOW_USER | SHOW_DOMAIN | SHOW_FOLDER },
+	{ "dav",  80,	SHOW_PATH | SHOW_PORT | SHOW_USER | SHOW_FOLDER },
+	{ "davs", 443,	SHOW_PATH | SHOW_PORT | SHOW_USER | SHOW_FOLDER },
 	{ "obex", 0,	SHOW_DEVICE },
 	{ NULL,   0,	0 }
 };
@@ -143,11 +158,21 @@ G_DEFINE_TYPE(GigoloBookmarkEditDialog, gigolo_bookmark_edit_dialog, GTK_TYPE_DI
 static void gigolo_bookmark_edit_dialog_destroy(GtkObject *object)
 {
 	GigoloBookmarkEditDialogPrivate *priv = GIGOLO_BOOKMARK_EDIT_DIALOG_GET_PRIVATE(object);
+	GigoloBackendGVFS *backend;
+
+	backend = gigolo_window_get_backend(priv->parent);
+	if (backend != NULL && IS_GIGOLO_BACKEND_GVFS(backend) && priv->browse_host_signal_id > 0)
+	{
+		g_signal_handler_disconnect(gigolo_window_get_backend(priv->parent), priv->browse_host_signal_id);
+		priv->browse_host_signal_id = 0;
+	}
 
 	gtk_widget_destroy(priv->uri_entry);
 	gtk_widget_destroy(priv->uri_label);
 	gtk_widget_destroy(priv->host_entry);
 	gtk_widget_destroy(priv->host_label);
+	gtk_widget_destroy(priv->folder_entry);
+	gtk_widget_destroy(priv->folder_label);
 	gtk_widget_destroy(priv->port_label);
 	gtk_widget_destroy(priv->port_spin);
 	gtk_widget_destroy(priv->user_entry);
@@ -245,6 +270,24 @@ gint gigolo_bookmark_edit_dialog_run(GigoloBookmarkEditDialog *dialog)
 					gtk_widget_grab_focus(priv->uri_entry);
 				}
 			}
+			if (! error && gtk_widget_get_parent(priv->path_entry) != NULL)
+			{
+				tmp = gtk_entry_get_text(GTK_ENTRY(priv->path_entry));
+				if (tmp[0] == '/')
+				{	/* remove leading slashes */
+					gtk_entry_set_text(GTK_ENTRY(priv->path_entry), tmp + 1);
+
+				}
+			}
+			if (! error && gtk_widget_get_parent(priv->share_entry) != NULL)
+			{
+				tmp = gtk_entry_get_text(GTK_ENTRY(priv->share_entry));
+				if (tmp[0] == '/')
+				{	/* remove leading slashes */
+					gtk_entry_set_text(GTK_ENTRY(priv->share_entry), tmp + 1);
+
+				}
+			}
 			if (! error)
 				break;
 		}
@@ -393,6 +436,10 @@ static void init_values(GigoloBookmarkEditDialog *dialog)
 		gtk_entry_set_text(GTK_ENTRY(priv->user_entry), user);
 		g_free(user);
 	}
+	/* Folder */
+	tmp = gigolo_bookmark_get_folder(priv->bookmark_init);
+	if (tmp != NULL)
+		gtk_entry_set_text(GTK_ENTRY(priv->folder_entry), tmp);
 	/* Share */
 	tmp = gigolo_bookmark_get_share(priv->bookmark_init);
 	if (tmp != NULL)
@@ -401,6 +448,10 @@ static void init_values(GigoloBookmarkEditDialog *dialog)
 	tmp = gigolo_bookmark_get_domain(priv->bookmark_init);
 	if (tmp != NULL)
 		gtk_entry_set_text(GTK_ENTRY(priv->domain_entry), tmp);
+	/* Path */
+	tmp = gigolo_bookmark_get_path(priv->bookmark_init);
+	if (tmp != NULL)
+		gtk_entry_set_text(GTK_ENTRY(priv->path_entry), tmp);
 	/* Port */
 	port = gigolo_bookmark_get_port(priv->bookmark_init);
 
@@ -445,6 +496,11 @@ static void setup_for_type(GigoloBookmarkEditDialog *dialog)
 		gtk_container_remove(GTK_CONTAINER(priv->table), priv->host_label);
 		gtk_container_remove(GTK_CONTAINER(priv->table), priv->host_entry);
 	}
+	if (gtk_widget_get_parent(priv->folder_entry) != NULL)
+	{
+		gtk_container_remove(GTK_CONTAINER(priv->table), priv->folder_label);
+		gtk_container_remove(GTK_CONTAINER(priv->table), priv->folder_entry);
+	}
 	if (gtk_widget_get_parent(priv->port_spin) != NULL)
 	{
 		gtk_container_remove(GTK_CONTAINER(priv->table), priv->port_label);
@@ -460,6 +516,11 @@ static void setup_for_type(GigoloBookmarkEditDialog *dialog)
 		gtk_container_remove(GTK_CONTAINER(priv->table), priv->domain_label);
 		gtk_container_remove(GTK_CONTAINER(priv->table), priv->domain_entry);
 	}
+	if (gtk_widget_get_parent(priv->path_entry) != NULL)
+	{
+		gtk_container_remove(GTK_CONTAINER(priv->table), priv->path_label);
+		gtk_container_remove(GTK_CONTAINER(priv->table), priv->path_entry);
+	}
 	if (gtk_widget_get_parent(priv->share_combo) != NULL)
 	{
 		gtk_container_remove(GTK_CONTAINER(priv->table), priv->share_label);
@@ -525,6 +586,20 @@ static void setup_for_type(GigoloBookmarkEditDialog *dialog)
 
 			i++;
 		}
+		if (meth->flags & SHOW_PATH)
+		{
+			gtk_misc_set_alignment(GTK_MISC(priv->path_label), 0.0, 0.5);
+			gtk_widget_show(priv->path_label);
+			gtk_table_attach(GTK_TABLE(table), priv->path_label,
+					  0, 1, i, i+1, GTK_FILL, GTK_FILL, 0, 0);
+
+			gtk_label_set_mnemonic_widget(GTK_LABEL(priv->path_label), priv->path_entry);
+			gtk_widget_show(priv->path_entry);
+			gtk_table_attach(GTK_TABLE(table), priv->path_entry,
+					  1, 2, i, i+1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+
+			i++;
+		}
 	}
 
 	if (meth->flags & (SHOW_PORT | SHOW_DOMAIN | SHOW_USER))
@@ -551,6 +626,21 @@ static void setup_for_type(GigoloBookmarkEditDialog *dialog)
 			i++;
 		}
 
+		if (meth->flags & SHOW_FOLDER && priv->dialog_type != GIGOLO_BE_MODE_CONNECT)
+		{
+			gtk_misc_set_alignment(GTK_MISC(priv->folder_label), 0.0, 0.5);
+			gtk_widget_show(priv->folder_label);
+			gtk_table_attach(GTK_TABLE(table), priv->folder_label,
+					  0, 1, i, i+1, GTK_FILL, GTK_FILL, 0, 0);
+
+			gtk_label_set_mnemonic_widget(GTK_LABEL(priv->folder_label), priv->folder_entry);
+			gtk_widget_show(priv->folder_entry);
+			gtk_table_attach(GTK_TABLE(table), priv->folder_entry,
+					  1, 2, i, i+1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+
+			i++;
+		}
+
 		if (meth->flags & SHOW_DOMAIN)
 		{
 			gtk_misc_set_alignment(GTK_MISC(priv->domain_label), 0.0, 0.5);
@@ -634,8 +724,7 @@ static void fill_method_combo_box(GigoloBookmarkEditDialog *dialog)
 		}
 		scheme = gigolo_describe_scheme((methods[i].scheme != NULL) ? methods[i].scheme : "custom");
 
-		gtk_list_store_append(store, &iter);
-		gtk_list_store_set(store, &iter,
+		gtk_list_store_insert_with_values(store, &iter, -1,
 			COLUMN_INDEX, i,
 			COLUMN_VISIBLE, visible,
 			COLUMN_DESC, scheme,
@@ -707,8 +796,12 @@ static void update_bookmark(GigoloBookmarkEditDialog *dialog)
 
 		tmp = gtk_entry_get_text(GTK_ENTRY(priv->host_entry));
 		gigolo_bookmark_set_host(priv->bookmark_update, tmp);
+		tmp = gtk_entry_get_text(GTK_ENTRY(priv->folder_entry));
+		gigolo_bookmark_set_folder(priv->bookmark_update, tmp);
 		tmp = gtk_entry_get_text(GTK_ENTRY(priv->user_entry));
 		gigolo_bookmark_set_user(priv->bookmark_update, tmp);
+		tmp = gtk_entry_get_text(GTK_ENTRY(priv->path_entry));
+		gigolo_bookmark_set_path(priv->bookmark_update, tmp);
 		tmp = gtk_entry_get_text(GTK_ENTRY(priv->domain_entry));
 		gigolo_bookmark_set_domain(priv->bookmark_update, tmp);
 		tmp = gtk_entry_get_text(GTK_ENTRY(priv->share_entry));
@@ -743,8 +836,9 @@ static void gigolo_bookmark_edit_dialog_set_property(GObject *object, guint prop
 		const gchar *title;
 		const gchar *stock_id;
 		const gchar *button_stock_id;
+		gint mode = g_value_get_int(value);
 
-		switch (g_value_get_int(value))
+		switch (mode)
 		{
 			case GIGOLO_BE_MODE_CREATE:
 			{
@@ -776,6 +870,7 @@ static void gigolo_bookmark_edit_dialog_set_property(GObject *object, guint prop
 		gtk_window_set_title(GTK_WINDOW(dialog), title);
 		gtk_window_set_icon_name(GTK_WINDOW(dialog), stock_id);
 		gtk_dialog_add_buttons(GTK_DIALOG(dialog), button_stock_id, GTK_RESPONSE_OK, NULL);
+		priv->dialog_type = mode;
 
 		setup_for_type(dialog);
         break;
@@ -787,11 +882,27 @@ static void gigolo_bookmark_edit_dialog_set_property(GObject *object, guint prop
 }
 
 
+static void browse_host_finished_cb(G_GNUC_UNUSED GigoloBackendGVFS *bnd, GSList *shares,
+									GigoloBookmarkEditDialog *dialog)
+{
+	GigoloBookmarkEditDialogPrivate *priv = GIGOLO_BOOKMARK_EDIT_DIALOG_GET_PRIVATE(dialog);
+
+	if (shares != NULL)
+	{
+		GSList *node;
+		for (node = shares; node != NULL; node = node->next)
+		{
+			gtk_combo_box_append_text(GTK_COMBO_BOX(priv->share_combo), node->data);
+		}
+	}
+	gtk_widget_set_sensitive(priv->share_button, TRUE);
+}
+
+
 static void share_button_clicked_cb(GtkWidget *btn, GigoloBookmarkEditDialog *dialog)
 {
 	GigoloBookmarkEditDialogPrivate *priv = GIGOLO_BOOKMARK_EDIT_DIALOG_GET_PRIVATE(dialog);
 	const gchar *hostname;
-	gchar **shares;
 
 	gtk_list_store_clear(GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(priv->share_combo))));
 
@@ -801,20 +912,8 @@ static void share_button_clicked_cb(GtkWidget *btn, GigoloBookmarkEditDialog *di
 
 	gtk_widget_set_sensitive(btn, FALSE);
 
-	/* FIXME for now we ignore username and domain when browsing for shares as this is not yet
-	 * supported by GVfs (1.0.x). */
-	shares = gigolo_backend_gvfs_get_smb_shares(hostname, NULL, NULL);
-
-	if (shares != NULL)
-	{
-		guint i, len = g_strv_length(shares);
-		for (i = 0; i < len; i++)
-		{
-			gtk_combo_box_append_text(GTK_COMBO_BOX(priv->share_combo), shares[i]);
-		}
-		g_strfreev(shares);
-	}
-	gtk_widget_set_sensitive(btn, TRUE);
+	gigolo_backend_gvfs_browse_host(gigolo_window_get_backend(priv->parent),
+		GTK_WINDOW(priv->parent), hostname);
 }
 
 
@@ -838,6 +937,9 @@ static void gigolo_bookmark_edit_dialog_init(GigoloBookmarkEditDialog *dialog)
 	GtkCellRenderer *renderer;
 	GigoloBookmarkEditDialogPrivate *priv = GIGOLO_BOOKMARK_EDIT_DIALOG_GET_PRIVATE(dialog);
 
+	priv->browse_host_signal_id = 0;
+	priv->dialog_type = GIGOLO_BE_MODE_EDIT;
+
 	gtk_container_set_border_width(GTK_CONTAINER(dialog), 5);
 	gtk_box_set_spacing(GTK_BOX(gigolo_dialog_get_content_area(GTK_DIALOG(dialog))), 2);
 
@@ -896,6 +998,8 @@ static void gigolo_bookmark_edit_dialog_init(GigoloBookmarkEditDialog *dialog)
 
 	priv->uri_entry = gtk_entry_new();
 	priv->host_entry = gtk_entry_new();
+	priv->folder_entry = gtk_entry_new();
+	priv->path_entry = gtk_entry_new();
 	priv->port_spin = gtk_spin_button_new_with_range(0, 65535, 1);
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(priv->port_spin), 0.0);
 	gtk_widget_set_tooltip_text(priv->port_spin, _("Set the port to 0 to use the default port"));
@@ -906,6 +1010,8 @@ static void gigolo_bookmark_edit_dialog_init(GigoloBookmarkEditDialog *dialog)
 
 	priv->uri_label = gtk_label_new_with_mnemonic(_("_Location (URI):"));
 	priv->host_label = gtk_label_new_with_mnemonic(_("_Server:"));
+	priv->folder_label = gtk_label_new_with_mnemonic(_("_Folder:"));
+	priv->path_label = gtk_label_new_with_mnemonic(_("_Path:"));
 	priv->user_label = gtk_label_new_with_mnemonic(_("_User Name:"));
 	priv->information_label = gtk_label_new(_("Optional information:"));
 	priv->port_label = gtk_label_new_with_mnemonic(_("_Port:"));
@@ -914,6 +1020,8 @@ static void gigolo_bookmark_edit_dialog_init(GigoloBookmarkEditDialog *dialog)
 
 	gtk_entry_set_activates_default(GTK_ENTRY(priv->name_entry), TRUE);
 	gtk_entry_set_activates_default(GTK_ENTRY(priv->uri_entry), TRUE);
+	gtk_entry_set_activates_default(GTK_ENTRY(priv->folder_entry), TRUE);
+	gtk_entry_set_activates_default(GTK_ENTRY(priv->path_entry), TRUE);
 	gtk_entry_set_activates_default(GTK_ENTRY(priv->host_entry), TRUE);
 	gtk_entry_set_activates_default(GTK_ENTRY(priv->port_spin), TRUE);
 	gtk_entry_set_activates_default(GTK_ENTRY(priv->user_entry), TRUE);
@@ -932,6 +1040,10 @@ static void gigolo_bookmark_edit_dialog_init(GigoloBookmarkEditDialog *dialog)
 	g_object_ref(priv->uri_label);
 	g_object_ref(priv->host_entry);
 	g_object_ref(priv->host_label);
+	g_object_ref(priv->folder_entry);
+	g_object_ref(priv->folder_label);
+	g_object_ref(priv->path_entry);
+	g_object_ref(priv->path_label);
 	g_object_ref(priv->port_label);
 	g_object_ref(priv->port_spin);
 	g_object_ref(priv->user_entry);
@@ -949,13 +1061,20 @@ static void gigolo_bookmark_edit_dialog_init(GigoloBookmarkEditDialog *dialog)
 
 GtkWidget *gigolo_bookmark_edit_dialog_new(GigoloWindow *parent, GigoloBookmarkEditDialogMode mode)
 {
-	GigoloBookmarkEditDialog *dialog = g_object_new(GIGOLO_BOOKMARK_EDIT_DIALOG_TYPE,
+	GigoloBookmarkEditDialog *dialog;
+	GigoloBookmarkEditDialogPrivate *priv;
+
+	dialog = g_object_new(GIGOLO_BOOKMARK_EDIT_DIALOG_TYPE,
 		"transient-for", parent,
 		"mode", mode,
 		NULL);
-	GigoloBookmarkEditDialogPrivate *priv = GIGOLO_BOOKMARK_EDIT_DIALOG_GET_PRIVATE(dialog);
+
+	priv = GIGOLO_BOOKMARK_EDIT_DIALOG_GET_PRIVATE(dialog);
 	priv->parent = parent;
 
+	priv->browse_host_signal_id = g_signal_connect(gigolo_window_get_backend(parent),
+		"browse-host-finished", G_CALLBACK(browse_host_finished_cb), dialog);
+
 	return GTK_WIDGET(dialog);
 }
 
@@ -963,15 +1082,12 @@ GtkWidget *gigolo_bookmark_edit_dialog_new(GigoloWindow *parent, GigoloBookmarkE
 GtkWidget *gigolo_bookmark_edit_dialog_new_with_bookmark(GigoloWindow *parent,
 		GigoloBookmarkEditDialogMode mode, GigoloBookmark *bookmark)
 {
-	GigoloBookmarkEditDialog *dialog = g_object_new(GIGOLO_BOOKMARK_EDIT_DIALOG_TYPE,
-		"transient-for", parent,
-		"bookmark-init", bookmark,
-		"mode", mode,
-		NULL);
-	GigoloBookmarkEditDialogPrivate *priv = GIGOLO_BOOKMARK_EDIT_DIALOG_GET_PRIVATE(dialog);
-	priv->parent = parent;
+	GtkWidget *dialog;
 
-	return GTK_WIDGET(dialog);
+	dialog = gigolo_bookmark_edit_dialog_new(parent, mode);
+	g_object_set(dialog, "bookmark-init", bookmark, NULL);
+
+	return dialog;
 }
 
 
