@@ -182,9 +182,13 @@ static gboolean pw_dialog_input_is_valid(GigoloMountOperation *operation)
 	GigoloMountOperationPrivate *priv = GIGOLO_MOUNT_OPERATION_GET_PRIVATE(operation);
 	gboolean is_valid = TRUE;
 
-	is_valid = entry_has_input(priv->username_entry) &&
-			entry_has_input(priv->domain_entry) &&
-			entry_has_input(priv->password_entry);
+	/* We don't require password to be non-empty here
+	* since there are situations where it is not needed,
+	* see bug 578365.
+	* We may add a way for the backend to specify that it
+	* definitively needs a password.
+	*/
+	is_valid = entry_has_input(priv->username_entry) && entry_has_input(priv->domain_entry);
 
 	return is_valid;
 }
