@@ -26,6 +26,7 @@
 
 #include "common.h"
 #include "compat.h"
+#include "bookmark.h"
 #include "settings.h"
 #include "backendgvfs.h"
 #include "preferencesdialog.h"
@@ -192,7 +193,7 @@ static GtkWidget *add_view_mode_combo(GigoloSettings *settings, const gchar *pro
 
 	widget = gtk_combo_box_new_text();
 	gtk_combo_box_append_text(GTK_COMBO_BOX(widget), _("Symbols"));
-	gtk_combo_box_append_text(GTK_COMBO_BOX(widget), _("Detailed"));
+	gtk_combo_box_append_text(GTK_COMBO_BOX(widget), _("Detailed List"));
 
 	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), value);
 
@@ -414,9 +415,8 @@ static void set_settings(GigoloPreferencesDialog *dialog, GigoloSettings *settin
 	g_signal_connect(tmp_box, "toggled", G_CALLBACK(check_button_toggle_sensitive_cb), checkbox);
 	check_button_toggle_sensitive_cb(GTK_TOGGLE_BUTTON(tmp_box), checkbox);
 
-	checkbox = add_check_button(settings, "show-panel", _("Show 'Browse Network' side panel"));
-	gtk_widget_set_tooltip_text(checkbox, _("Whether to show a side panel for browsing the local network for available Samba shares"));
-	gtk_widget_set_sensitive(checkbox, gigolo_backend_gvfs_is_scheme_supported("smb"));
+	checkbox = add_check_button(settings, "show-panel", _("Show side panel"));
+	gtk_widget_set_tooltip_text(checkbox, _("Whether to show a side panel for browsing the local network for available Samba/Windows shares and a bookmark list"));
 	gtk_box_pack_start(GTK_BOX(frame_vbox), checkbox, FALSE, FALSE, 0);
 
 	checkbox = add_check_button(settings, "show-autoconnect-errors", _("Show auto-connect error messages"));
@@ -426,7 +426,7 @@ static void set_settings(GigoloPreferencesDialog *dialog, GigoloSettings *settin
 	hbox = gtk_hbox_new(FALSE, 6);
 	gtk_box_pack_start(GTK_BOX(frame_vbox), hbox, FALSE, FALSE, 0);
 
-	label4 = gtk_label_new_with_mnemonic(_("_View Mode"));
+	label4 = gtk_label_new_with_mnemonic(_("_Connection List Mode"));
 	gtk_misc_set_alignment(GTK_MISC(label4), 0.0f, 0.5f);
 	gtk_box_pack_start(GTK_BOX(hbox), label4, FALSE, FALSE, 0);
 
