@@ -27,9 +27,9 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "bookmark.h"
 #include "settings.h"
 #include "common.h"
-#include "bookmark.h"
 
 
 typedef struct _GigoloSettingsPrivate			GigoloSettingsPrivate;
@@ -51,6 +51,7 @@ struct _GigoloSettingsPrivate
 	gint		toolbar_orientation;
 	gint		view_mode;
 	gboolean	show_panel;
+	guint		last_panel_page;
 	gboolean	show_autoconnect_errors;
 
 	gchar		*file_manager;
@@ -87,6 +88,7 @@ enum
 	PROP_TOOLBAR_ORIENTATION,
 	PROP_VIEW_MODE,
 	PROP_SHOW_PANEL,
+	PROP_LAST_PANEL_PAGE,
 	PROP_SHOW_AUTOCONNECT_ERRORS
 };
 
@@ -131,6 +133,9 @@ static void gigolo_settings_set_property(GObject *object, guint prop_id, const G
 	case PROP_SHOW_PANEL:
 		priv->show_panel = g_value_get_boolean(value);
 		break;
+	case PROP_LAST_PANEL_PAGE:
+		priv->last_panel_page = g_value_get_uint(value);
+		break;
 	case PROP_SHOW_AUTOCONNECT_ERRORS:
 		priv->show_autoconnect_errors = g_value_get_boolean(value);
 		break;
@@ -179,6 +184,9 @@ static void gigolo_settings_get_property(GObject *object, guint prop_id, GValue
 	case PROP_SHOW_PANEL:
 		g_value_set_boolean(value, priv->show_panel);
 		break;
+	case PROP_LAST_PANEL_PAGE:
+		g_value_set_uint(value, priv->last_panel_page);
+		break;
 	case PROP_SHOW_AUTOCONNECT_ERRORS:
 		g_value_set_boolean(value, priv->show_autoconnect_errors);
 		break;
@@ -278,6 +286,14 @@ static void gigolo_settings_class_init(GigoloSettingsClass *klass)
 									"Whether to show the side panel",
 									TRUE,
 									G_PARAM_READWRITE));
+	g_object_class_install_property(gobject_class,
+									PROP_LAST_PANEL_PAGE,
+									g_param_spec_uint(
+									"last-panel-page",
+									"last-panel-page",
+									"Last displayed panel page",
+									0, G_MAXUINT, 0,
+									G_PARAM_READWRITE));
 	g_object_class_install_property(gobject_class,
 									PROP_SHOW_AUTOCONNECT_ERRORS,
 									g_param_spec_boolean(
@@ -411,6 +427,7 @@ static void write_settings_config(GigoloSettings *settings)
 	g_key_file_set_integer(k, SECTION_UI, "toolbar_orientation", priv->toolbar_orientation);
 	g_key_file_set_integer(k, SECTION_UI, "view_mode", priv->view_mode);
 	g_key_file_set_boolean(k, SECTION_UI, "show_panel", priv->show_panel);
+	g_key_file_set_integer(k, SECTION_UI, "last_panel_page", priv->last_panel_page);
 	g_key_file_set_boolean(k, SECTION_UI, "show_autoconnect_errors", priv->show_autoconnect_errors);
 
 	write_data(k, priv->config_filename);
@@ -441,10 +458,12 @@ static void write_settings_bookmarks(GigoloSettings *settings)
 		{
 			name = gigolo_bookmark_get_name(bm);
 			set_setting_string(k, name, "host", gigolo_bookmark_get_host(bm));
+			set_setting_string(k, name, "folder", gigolo_bookmark_get_folder(bm));
 			set_setting_string(k, name, "user", gigolo_bookmark_get_user(bm));
 			set_setting_string(k, name, "scheme", gigolo_bookmark_get_scheme(bm));
 			set_setting_string(k, name, "share", gigolo_bookmark_get_share(bm));
 			set_setting_string(k, name, "domain", gigolo_bookmark_get_domain(bm));
+			set_setting_string(k, name, "path", gigolo_bookmark_get_path(bm));
 			set_setting_int(k, name, "port", gigolo_bookmark_get_port(bm));
 			set_setting_int(k, name, "autoconnect", gigolo_bookmark_get_autoconnect(bm));
 		}
@@ -516,6 +535,7 @@ static void load_settings_read_config(GigoloSettingsPrivate *priv)
 	priv->show_in_systray = get_setting_boolean(k, SECTION_UI, "show_in_systray", TRUE);
 	priv->start_in_systray = get_setting_boolean(k, SECTION_UI, "start_in_systray", FALSE);
 	priv->show_panel = get_setting_boolean(k, SECTION_UI, "show_panel", FALSE);
+	priv->last_panel_page = get_setting_int(k, SECTION_UI, "last_panel_page", 0);
 	priv->show_autoconnect_errors = get_setting_boolean(k, SECTION_UI, "show_autoconnect_errors", TRUE);
 	priv->show_toolbar = get_setting_boolean(k, SECTION_UI, "show_toolbar", TRUE);
 	priv->toolbar_style = get_setting_int(k, SECTION_UI, "toolbar_style", -1);
@@ -552,7 +572,7 @@ static void load_settings_read_bookmarks(GigoloSettingsPrivate *priv)
 	GError *error = NULL;
 	gsize len, i;
 	gchar **groups;
-	gchar *scheme, *host, *user, *domain, *share;
+	gchar *scheme, *host, *user, *domain, *share, *folder, *path;
 	gint port;
 	gboolean autoconnect;
 	GigoloBookmark *bm;
@@ -571,6 +591,8 @@ static void load_settings_read_bookmarks(GigoloSettingsPrivate *priv)
 	{
 		scheme = get_setting_string(k, groups[i], "scheme", "");
 		host = get_setting_string(k, groups[i], "host", "");
+		folder = get_setting_string(k, groups[i], "folder", "");
+		path = get_setting_string(k, groups[i], "path", "");
 		user = get_setting_string(k, groups[i], "user", "");
 		domain = get_setting_string(k, groups[i], "domain", "");
 		share = get_setting_string(k, groups[i], "share", "");
@@ -582,10 +604,14 @@ static void load_settings_read_bookmarks(GigoloSettingsPrivate *priv)
 		gigolo_bookmark_set_scheme(bm, scheme);
 		if (NZV(host))
 			gigolo_bookmark_set_host(bm, host);
+		if (NZV(folder))
+			gigolo_bookmark_set_folder(bm, folder);
 		if (NZV(user))
 			gigolo_bookmark_set_user(bm, user);
 		if (NZV(domain))
 			gigolo_bookmark_set_domain(bm, domain);
+		if (NZV(path))
+			gigolo_bookmark_set_path(bm, path);
 		if (NZV(share))
 			gigolo_bookmark_set_share(bm, share);
 		gigolo_bookmark_set_port(bm, port);
@@ -595,6 +621,8 @@ static void load_settings_read_bookmarks(GigoloSettingsPrivate *priv)
 
 		g_free(scheme);
 		g_free(host);
+		g_free(folder);
+		g_free(path);
 		g_free(user);
 		g_free(domain);
 		g_free(share);
@@ -763,3 +791,31 @@ gboolean gigolo_settings_has_file_manager(GigoloSettings *settings)
 
 	return NZV(priv->file_manager);
 }
+
+
+GigoloBookmark *gigolo_settings_get_bookmark_by_uri(GigoloSettings *settings, const gchar *uri)
+{
+	GigoloBookmarkList *bml;
+	GigoloBookmark *bm = NULL;
+	gboolean found = FALSE;
+	gchar *tmp_uri;
+	guint i;
+
+	g_return_val_if_fail(settings != NULL, FALSE);
+	g_return_val_if_fail(uri != NULL, FALSE);
+
+	bml = gigolo_settings_get_bookmarks(settings);
+
+	for (i = 0; i < bml->len && ! found; i++)
+	{
+		bm = g_ptr_array_index(bml, i);
+		tmp_uri = gigolo_bookmark_get_uri_escaped(bm);
+		if (gigolo_str_equal(uri, tmp_uri))
+			found = TRUE;
+
+		g_free(tmp_uri);
+	}
+	return (found) ? bm : NULL;
+}
+
+
