@@ -38,6 +38,7 @@
 #include "preferencesdialog.h"
 #include "mountdialog.h"
 #include "browsenetworkpanel.h"
+#include "bookmarkpanel.h"
 
 
 typedef struct _GigoloWindowPrivate			GigoloWindowPrivate;
@@ -59,6 +60,8 @@ struct _GigoloWindowPrivate
 	GtkWidget		*hbox_view;
 
 	GtkWidget		*browse_panel;
+	GtkWidget		*bookmark_panel;
+	GtkWidget		*notebook_panel;
 	GtkWidget		*treeview;
 	GtkWidget		*iconview;
 	GtkWidget		*swin_treeview;
@@ -130,6 +133,9 @@ static void gigolo_window_destroy(GigoloWindow *window)
 
 		gigolo_settings_set_geometry(priv->settings, geo, 5);
 	}
+	g_object_set(priv->settings, "last-panel-page",
+		gtk_notebook_get_current_page(GTK_NOTEBOOK(priv->notebook_panel)), NULL);
+
 	gtk_widget_destroy(priv->tree_popup_menu);
 	gtk_widget_destroy(priv->systray_icon_popup_menu);
 	gtk_widget_destroy(priv->swin_treeview);
@@ -139,6 +145,7 @@ static void gigolo_window_destroy(GigoloWindow *window)
 	g_object_unref(priv->systray_icon);
 	g_object_unref(priv->systray_icon_popup_menu);
 	g_object_unref(priv->backend_gvfs);
+	priv->backend_gvfs = NULL;
 
 	gtk_widget_destroy(GTK_WIDGET(window));
 
@@ -211,50 +218,21 @@ static void get_selected_iter(GigoloWindow *window, GtkTreeIter *iter)
 	}
 	else
 	{
-		GList *l, *items = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(priv->iconview));
+		GList *items = gtk_icon_view_get_selected_items(GTK_ICON_VIEW(priv->iconview));
 		GtkTreeModel *model = gtk_icon_view_get_model(GTK_ICON_VIEW(priv->iconview));
 
-		for (l = items; l != NULL; l = l->next)
+		if (items != NULL)
 		{
-			gtk_tree_model_get_iter(model, iter, l->data);
-			/* the selection mode is SINGLE, so the list should never have more than one entry */
-			break;
+			/* the selection mode is SINGLE, so the list should never have more than one entry
+			 * and we simply choose the first item */
+			gtk_tree_model_get_iter(model, iter, items->data);
 		}
-
 		g_list_foreach(items, (GFunc) gtk_tree_path_free, NULL);
 		g_list_free(items);
 	}
 }
 
 
-GigoloBookmark *gigolo_window_find_bookmark_by_uri(GigoloWindow *window, const gchar *uri)
-{
-	GigoloWindowPrivate *priv;
-	GigoloBookmarkList *bml;
-	GigoloBookmark *bm = NULL;
-	gboolean found = FALSE;
-	gchar *tmp_uri;
-	guint i;
-
-	g_return_val_if_fail(window != NULL, FALSE);
-	g_return_val_if_fail(uri != NULL, FALSE);
-
-	priv = GIGOLO_WINDOW_GET_PRIVATE(window);
-	bml = gigolo_settings_get_bookmarks(priv->settings);
-
-	for (i = 0; i < bml->len && ! found; i++)
-	{
-		bm = g_ptr_array_index(bml, i);
-		tmp_uri = gigolo_bookmark_get_uri_escaped(bm);
-		if (gigolo_str_equal(uri, tmp_uri))
-			found = TRUE;
-
-		g_free(tmp_uri);
-	}
-	return (found) ? bm : NULL;
-}
-
-
 void gigolo_window_mount_from_bookmark(GigoloWindow *window, GigoloBookmark *bookmark,
 									   gboolean show_dialog, gboolean show_errors)
 {
@@ -306,7 +284,7 @@ static void action_mount_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *windo
 			GIGOLO_WINDOW_COL_REF, &vol, -1);
 
 		if (ref_type == GIGOLO_WINDOW_REF_TYPE_VOLUME)
-			handled = gigolo_backend_gvfs_mount_volume(priv->backend_gvfs, vol);
+			handled = gigolo_backend_gvfs_mount_volume(priv->backend_gvfs, GTK_WINDOW(window), vol);
 	}
 
 	if (! handled)
@@ -390,7 +368,7 @@ static void action_unmount_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *win
 		{
 			gchar *uri;
 			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, NULL, &uri);
-			bm = gigolo_window_find_bookmark_by_uri(window, uri);
+			bm = gigolo_settings_get_bookmark_by_uri(priv->settings, uri);
 			if (bm != NULL && gigolo_bookmark_get_autoconnect(bm))
 			{	/* we don't want auto-connection to reconnect this bookmark right
 				   after we unmount it. */
@@ -398,7 +376,7 @@ static void action_unmount_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *win
 			}
 			g_free(uri);
 
-			gigolo_backend_gvfs_unmount_mount(priv->backend_gvfs, mnt);
+			gigolo_backend_gvfs_unmount_mount(priv->backend_gvfs, mnt, GTK_WINDOW(window));
 		}
 	}
 }
@@ -516,8 +494,14 @@ static void action_copy_uri_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *wi
 		if (gigolo_backend_gvfs_is_mount(mnt))
 		{
 			gchar *uri;
+			GigoloBookmark *b;
 
 			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, NULL, &uri);
+
+			b = gigolo_settings_get_bookmark_by_uri(priv->settings, uri);
+			if (b != NULL)
+				setptr(uri, g_build_filename(uri, gigolo_bookmark_get_folder(b), NULL));
+
 			gtk_clipboard_set_text(gtk_clipboard_get(gdk_atom_intern("CLIPBOARD", FALSE)), uri, -1);
 
 			g_free(uri);
@@ -542,52 +526,30 @@ static void action_open_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *window
 		gtk_tree_model_get(model, &iter, GIGOLO_WINDOW_COL_REF, &mnt, -1);
 		if (gigolo_backend_gvfs_is_mount(mnt))
 		{
-#if 1
 			GError *error = NULL;
 			gchar *uri;
 			gchar *file_manager;
 			gchar *cmd;
+			GigoloBookmark *b;
 
 			file_manager = gigolo_settings_get_string(priv->settings, "file-manager");
 			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, NULL, &uri);
-			cmd = g_strconcat(file_manager, " ", uri, NULL);
+			b = gigolo_settings_get_bookmark_by_uri(priv->settings, uri);
+			if (b != NULL)
+				setptr(uri, g_build_filename(uri, gigolo_bookmark_get_folder(b), NULL));
+			/* escape spaces and similar */
+			setptr(uri, g_uri_unescape_string(uri, G_URI_RESERVED_CHARS_ALLOWED_IN_USERINFO));
 
+			cmd = g_strconcat(file_manager, " \"", uri, "\"", NULL);
 			if (! g_spawn_command_line_async(cmd, &error))
 			{
-				verbose(error->message);
+				verbose("%s", error->message);
 				g_error_free(error);
 			}
 
 			g_free(cmd);
 			g_free(file_manager);
 			g_free(uri);
-#else
-/*
-			GFile *file = g_mount_get_root(mnt);
-			gchar *path = g_file_get_path(file);
-
-			if (path != NULL)
-			{
-				gchar *cmd = g_strconcat("xdg-open ", path, NULL);
-				g_spawn_command_line_async(cmd, NULL);
-				g_free(cmd);
-				g_free(path);
-			}
-			else
-			{
-				// FIXME make the open command configurable or find a better solution
-				//       maybe gtk_show_uri() ?
-				GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(window),
-							GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
-							_("Non-local mountpoints can't be opened."));
-				gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), _("(not yet implemented)"));
-				gtk_dialog_run(GTK_DIALOG (dialog));
-				gtk_widget_destroy(dialog);
-				verbose("Non-local mountpoints can't be opened.");
-			}
-			g_object_unref(file);
-*/
-#endif
 		}
 	}
 }
@@ -611,10 +573,11 @@ static gboolean iter_is_bookmark(GigoloWindow *window, GtkTreeModel *model, GtkT
 	{
 		gchar *uri;
 		gboolean found = FALSE;
+		GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 
 		gigolo_backend_gvfs_get_name_and_uri_from_mount(ref, NULL, &uri);
 
-		found = (gigolo_window_find_bookmark_by_uri(window, uri) != NULL);
+		found = (gigolo_settings_get_bookmark_by_uri(priv->settings, uri) != NULL);
 
 		g_free(uri);
 		return found;
@@ -827,6 +790,10 @@ void gigolo_window_update_bookmarks(GigoloWindow *window)
 
 	/* writing to the 'settings' property will update the menus */
 	g_object_set(priv->action_bookmarks, "settings", priv->settings, NULL);
+	g_object_set(priv->bookmark_panel, "settings", priv->settings, NULL);
+
+	/* update the popup menu items */
+	tree_selection_changed_cb(NULL, window);
 }
 
 
@@ -888,7 +855,7 @@ static void action_create_bookmark_cb(G_GNUC_UNUSED GtkAction *button, GigoloWin
 
 			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, &name, &uri);
 
-			if (gigolo_window_find_bookmark_by_uri(window, uri) == NULL)
+			if (gigolo_settings_get_bookmark_by_uri(priv->settings, uri) == NULL)
 			{
 				GigoloBookmark *bm = gigolo_bookmark_new_from_uri(name, uri);
 				if (gigolo_bookmark_is_valid(bm))
@@ -928,7 +895,7 @@ static void gigolo_window_show_side_panel(GigoloWindow *window, gboolean show)
 {
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 
-	if (show && gigolo_backend_gvfs_is_scheme_supported("smb"))
+	if (show)
 		gtk_widget_show(priv->hbox_pane);
 	else
 		gtk_widget_hide(priv->hbox_pane);
@@ -1027,15 +994,6 @@ static void view_mode_action_set_active(GigoloWindow *window, gint val)
 }
 
 
-static void action_set_sensitive(GigoloWindow *window, const gchar *name, gboolean set)
-{
-	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
-	GtkAction *action = gtk_action_group_get_action(priv->action_group, name);
-
-	gtk_action_set_sensitive(action, set);
-}
-
-
 static void gigolo_window_systray_notify_cb(GtkStatusIcon *sicon, GParamSpec *pspec, GtkWindow *window)
 {
 	const gchar *name;
@@ -1289,7 +1247,7 @@ static void create_tree_view(GigoloWindow *window)
 		column = gtk_tree_view_column_new_with_attributes(NULL, renderer,
 			"gicon", GIGOLO_WINDOW_COL_PIXBUF, NULL);
 		gtk_tree_view_column_set_sort_indicator(column, FALSE);
-		gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
+		gtk_tree_view_column_set_resizable(column, TRUE);
 		gtk_tree_view_append_column(GTK_TREE_VIEW(priv->treeview), column);
 	}
 
@@ -1298,7 +1256,7 @@ static void create_tree_view(GigoloWindow *window)
 		_("Connected"), renderer, "active", GIGOLO_WINDOW_COL_IS_MOUNTED, NULL);
 	gtk_tree_view_column_set_sort_indicator(column, TRUE);
 	gtk_tree_view_column_set_sort_column_id(column, GIGOLO_WINDOW_COL_IS_MOUNTED);
-	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(priv->treeview), column);
 	g_signal_connect(renderer, "toggled", G_CALLBACK(tree_mounted_col_toggled_cb), window);
 
@@ -1307,7 +1265,7 @@ static void create_tree_view(GigoloWindow *window)
 		_("Service Type"), renderer, "text", GIGOLO_WINDOW_COL_SCHEME, NULL);
 	gtk_tree_view_column_set_sort_indicator(column, TRUE);
 	gtk_tree_view_column_set_sort_column_id(column, GIGOLO_WINDOW_COL_SCHEME);
-	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(priv->treeview), column);
 
 	renderer = gtk_cell_renderer_text_new();
@@ -1315,7 +1273,7 @@ static void create_tree_view(GigoloWindow *window)
 		_("Name"), renderer, "text", GIGOLO_WINDOW_COL_NAME, NULL);
 	gtk_tree_view_column_set_sort_indicator(column, TRUE);
 	gtk_tree_view_column_set_sort_column_id(column, GIGOLO_WINDOW_COL_NAME);
-	gtk_tree_view_column_set_resizable(GTK_TREE_VIEW_COLUMN(column), TRUE);
+	gtk_tree_view_column_set_resizable(column, TRUE);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(priv->treeview), column);
 
 	gtk_tree_view_set_model(GTK_TREE_VIEW(priv->treeview), GTK_TREE_MODEL(priv->store));
@@ -1371,6 +1329,60 @@ static void create_icon_view(GigoloWindow *window)
 }
 
 
+static GtkWidget *gigolo_window_create_panel(GigoloWindow *window)
+{
+	GtkWidget *panel_pane, *label;
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+
+	panel_pane = gtk_hpaned_new();
+	gtk_paned_set_position(GTK_PANED(panel_pane), 200);
+
+	priv->notebook_panel = gtk_notebook_new();
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(priv->notebook_panel), GTK_POS_LEFT);
+	gtk_widget_show(priv->notebook_panel);
+
+	label = gtk_label_new(_("Bookmarks"));
+	gtk_label_set_angle(GTK_LABEL(label), 90.0);
+	gtk_widget_show(label);
+
+	priv->bookmark_panel = gigolo_bookmark_panel_new(window);
+	gtk_widget_show(priv->bookmark_panel);
+	gtk_notebook_append_page(GTK_NOTEBOOK(priv->notebook_panel), priv->bookmark_panel, label);
+
+	label = gtk_label_new(_("Network"));
+	gtk_label_set_angle(GTK_LABEL(label), 90.0);
+	gtk_widget_show(label);
+
+	priv->browse_panel = gigolo_browse_network_panel_new(window);
+	gtk_widget_show(priv->browse_panel);
+	gtk_notebook_append_page(GTK_NOTEBOOK(priv->notebook_panel), priv->browse_panel, label);
+
+	priv->hbox_view = gtk_hbox_new(FALSE, 0);
+	priv->hbox_pane = gtk_hbox_new(FALSE, 0);
+
+	gtk_paned_add1(GTK_PANED(panel_pane), priv->hbox_pane);
+	gtk_paned_add2(GTK_PANED(panel_pane), priv->hbox_view);
+	gtk_box_pack_start(GTK_BOX(priv->hbox_pane), priv->notebook_panel, TRUE, TRUE, 0);
+
+	return panel_pane;
+}
+
+
+static void update_side_panel(GigoloWindow *window)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+
+	if (! gigolo_backend_gvfs_is_scheme_supported("smb"))
+		gtk_widget_destroy(priv->browse_panel);
+
+	if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(priv->notebook_panel)) < 2)
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(priv->notebook_panel), FALSE);
+
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(priv->notebook_panel),
+		gigolo_settings_get_integer(priv->settings, "last-panel-page"));
+}
+
+
 static void gigolo_window_init(GigoloWindow *window)
 {
 	GtkWidget *menubar, *panel_pane;
@@ -1404,7 +1416,7 @@ static void gigolo_window_init(GigoloWindow *window)
 	gtk_container_add(GTK_CONTAINER(priv->swin_iconview), priv->iconview);
 
 	/* Init the GVfs backend */
-	priv->backend_gvfs = gigolo_backend_gvfs_new(priv->store);
+	priv->backend_gvfs = gigolo_backend_gvfs_new();
 	g_signal_connect(priv->backend_gvfs, "mounts-changed", G_CALLBACK(mounts_changed_cb), window);
 	g_signal_connect(priv->backend_gvfs, "operation-failed",
 		G_CALLBACK(mount_operation_failed_cb), window);
@@ -1435,20 +1447,10 @@ static void gigolo_window_init(GigoloWindow *window)
 		"is-important", TRUE, NULL);
 
 	/* Panel */
-	panel_pane = gtk_hpaned_new();
-	gtk_paned_set_position(GTK_PANED(panel_pane), 200);
-
-	priv->browse_panel = gigolo_browse_network_panel_new(window);
-	gtk_widget_show(priv->browse_panel);
+	panel_pane = gigolo_window_create_panel(window);
 
 	/* Pack the widgets altogether */
 	priv->vbox = gtk_vbox_new(FALSE, 0);
-	priv->hbox_view = gtk_hbox_new(FALSE, 0);
-	priv->hbox_pane = gtk_hbox_new(FALSE, 0);
-
-	gtk_paned_add1(GTK_PANED(panel_pane), priv->hbox_pane);
-	gtk_paned_add2(GTK_PANED(panel_pane), priv->hbox_view);
-	gtk_box_pack_start(GTK_BOX(priv->hbox_pane), priv->browse_panel, TRUE, TRUE, 0);
 
 	gtk_box_pack_start(GTK_BOX(priv->vbox), menubar, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(priv->vbox), priv->toolbar, FALSE, FALSE, 0);
@@ -1488,6 +1490,8 @@ GtkWidget *gigolo_window_new(GigoloSettings *settings)
 
 	g_object_set(priv->action_bookmarks, "settings", settings, NULL);
 
+	g_object_set(priv->backend_gvfs, "parent", window, "store", priv->store, NULL);
+
 	gigolo_window_set_toolbar_style(GIGOLO_WINDOW(window),
 		gigolo_settings_get_integer(settings, "toolbar-style"));
 	gigolo_window_set_toolbar_orientation(GIGOLO_WINDOW(window),
@@ -1496,7 +1500,7 @@ GtkWidget *gigolo_window_new(GigoloSettings *settings)
 	state = gigolo_settings_get_boolean(settings, "show-panel");
 	gigolo_window_show_side_panel(GIGOLO_WINDOW(window), state);
 	toggle_action_set_active(GIGOLO_WINDOW(window), "ShowPanel", state);
-	action_set_sensitive(GIGOLO_WINDOW(window), "ShowPanel", gigolo_backend_gvfs_is_scheme_supported("smb"));
+	update_side_panel(GIGOLO_WINDOW(window));
 	/* Show Toolbar */
 	state = gigolo_settings_get_boolean(settings, "show-toolbar");
 	gigolo_window_show_toolbar(GIGOLO_WINDOW(window), state);
